options = (
    ('sentry-dsn',
     {'type': 'string',
      'default': '',
      'help': 'Sentry DSN (eg. https://public_key:secret_key@sentry.local/project_id?timeout=10 )',
      'group': 'sentry',
      'level': 5,
      }
     ),
)
